package com.muyu.service.impl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.active.domain.ActivityTeamInfo;
import com.muyu.active.domain.ActivityTeamOpenInfo;
import com.muyu.active.domain.ActivityTeamProductSkuInfo;
import com.muyu.active.domain.model.*;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.mapper.ActivityTeamInfoMapper;
import com.muyu.service.ActivityTeamInfoService;
import com.muyu.service.ActivityTeamOpenInfoService;
import com.muyu.service.ActivityTeamProductSkuInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 商品拼团信息Service业务层处理
 */
@Slf4j
@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper, ActivityTeamInfo> implements ActivityTeamInfoService {
    @Resource
    private ActivityTeamOpenInfoService activityTeamOpenInfoService;
    @Resource
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;

    @Override
    public TableDataInfo<ActivityTeamInfoListModel> list(ActivityTeamInfoQueryModel activityTeamInfoQueryModel) {
        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        //判断是否为空查询的条件 名称
        queryWrapper.like(StringUtils.isNotEmpty(
                activityTeamInfoQueryModel.getKeyWord()),
                ActivityTeamInfo::getName,
                activityTeamInfoQueryModel.getKeyWord()
        );
        //判断是否为空查询的条件 状态
        queryWrapper.like(StringUtils.isNotEmpty(
                activityTeamInfoQueryModel.getStatus()),
                ActivityTeamInfo::getStatus,
                activityTeamInfoQueryModel.getStatus()
                );
        //分页
        Page<ActivityTeamInfo> pages = this.page(activityTeamInfoQueryModel.bulidPage(),queryWrapper);
        List<ActivityTeamInfo> records = pages.getRecords();

        List<ActivityTeamInfoListModel> list = records.stream().map(activityTeamInfo -> ActivityTeamInfoListModel
                .domainBuild(activityTeamInfo, (activityTeamInfoListModelBuilder) -> {
                    Long teamOpenNum = activityTeamOpenInfoService.getTeamOpenNum(activityTeamInfo.getId());
                    Long teamInNum = activityTeamOpenInfoService.getTeamInNum(activityTeamInfo.getId());
                    ActivityTeamDiscountModel activityTeamDiscountModel = activityTeamProductSkuInfoService.discountPrice(activityTeamInfo.getId());
                    TeamStockModel stock = activityTeamProductSkuInfoService.getStocks(activityTeamInfo.getId());
                    return activityTeamInfoListModelBuilder
                            .addTeamNumber(teamInNum)
                            .openTeamNumber(teamOpenNum)
                            .attendNumber(teamOpenNum + teamInNum)
                            .teamPrice(activityTeamDiscountModel.getTeamPrice())
                            .productPrice(activityTeamDiscountModel.getProductPrice())
                            .teamStock(stock.getTeamStock())
                            .remainStock(stock.getRemainStock())
                            .build();
                })).toList();

        return TableDataInfo.<ActivityTeamInfoListModel>builder()
                .total(pages.getTotal())
                .rows(list)
                .build();
    }

    /**
     * 添加拼团信息
     */
    @Override
    public boolean addTeam(ActivityTeamInfoSaveModel activityTeamInfoSaveModel) {
        //调用拼团信息表的构造方法进行添加吧自己定义的添加实体类
        ActivityTeamInfo activityTeamInfo = ActivityTeamInfo.saveActivityTeamInfo(activityTeamInfoSaveModel);
        //用this调用save添加方法把返回值放进去
        boolean save = this.save(activityTeamInfo);
        //判断添加的时候，添加规格sku列表的数据，调用activityTeamInfoSaveModel的集合
        if (save){
            List<TeamSkuInfoModel> projectSkuInfoList = activityTeamInfoSaveModel.getProjectSkuInfoList();
            //利用Steam流遍历，把集合中的数据进行转换，转换成ActivityTeamProductSkuInfo对象，然后进行批量添加
            //批量添加方法
            activityTeamProductSkuInfoService.saveBatch(projectSkuInfoList.stream().map(projectSkuInfo -> ActivityTeamProductSkuInfo.TeamSkuInfoBuild(projectSkuInfo, (activityTeamProductSkuInfoBuilder) -> {
                LambdaQueryWrapper<ActivityTeamOpenInfo> queryWrapper = new LambdaQueryWrapper<>();
                //获取团队的ID
                queryWrapper.eq(ActivityTeamOpenInfo::getTeamId,activityTeamInfo.getId());
                //获取团队中商品的ID
                queryWrapper.eq(ActivityTeamOpenInfo::getProductId,projectSkuInfo.getProductId());
                long count = activityTeamOpenInfoService.count(queryWrapper);
                return activityTeamProductSkuInfoBuilder
                        //获取商品表的ID
                        .teamId(activityTeamInfo.getId())
                        //获取sku的剩余库存
                        .remainStock(projectSkuInfo.getTeamStock() - count)
                        .build();
            })).toList());
        }
        return save;
    }
    /**
     * 通过添加模型，去进行商品拼团活动的操作
     * @param activityTeamInfoAddModel 团购添加模型
     */
    @Override
    public void save(ActivityTeamInfoAddModel activityTeamInfoAddModel) {
        ActivityTeamInfo activityTeamInfo = ActivityTeamInfo.addModelBuild(activityTeamInfoAddModel);
        Assert.isTrue(this.save(activityTeamInfo), StringUtils.format("团购活动添加失败：[{}]", activityTeamInfoAddModel));
        List<ActivityTeamProductSkuAddModel> teamProductSkuAddModelList = activityTeamInfoAddModel.getActivityTeamProductSkuAddModelList();
        teamProductSkuAddModelList.forEach(activityTeamProductSkuAddModel -> activityTeamProductSkuAddModel.setTeamId(activityTeamInfo.getId()));
        this.activityTeamProductSkuInfoService.batchSave(teamProductSkuAddModelList);
    }
    /**
     * 根据ID查询商品的详情
     * @param id 团购活动ID
     * @return 团购详情
     */
    @Override
    public ActivityTeamDetailModel findDetailById(Long id) {
        //获取id
        ActivityTeamInfo activityTeamInfo = this.getById(id);
        //查询teamId集合
        List<ActivityTeamProductSkuModel> activityTeamProductSkuModelList = activityTeamProductSkuInfoService.findListByTeamId(id);
        return ActivityTeamDetailModel.findSkuSumList(activityTeamInfo,
                (teamInfoFindByIdRespModelBuilder) -> teamInfoFindByIdRespModelBuilder.projectSkuInfoAddReqList(activityTeamProductSkuModelList).build()
        );
    }
    /**
     * 修改团购活动信息
     * @param activityTeamInfoUpdModel 团购活动信息
     * @return
     */
    @Override
    @Transactional
    public void update(ActivityTeamInfoUpdModel activityTeamInfoUpdModel) {

        //判断活动状态是否是关闭状态
        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ActivityTeamInfo::getId,activityTeamInfoUpdModel.getId());
        ActivityTeamInfo activityTeamInfo = this.getOne(queryWrapper);
        if ("已开启".equals(activityTeamInfo.getStatus())){
            throw new ServiceException("活动状态开启,不能修改！！！");
        }
//        Assert.isTrue(this.updateById(activityTeamInfoUpdModel), StringUtils.format("修改失败：[{}]", activityTeamInfoUpdModel));
        //修改拼团活动表信息
        this.updateById(ActivityTeamInfo.updTeamInfoBuild(activityTeamInfoUpdModel));
        //根据根据前台传过来的skuIds集合，批量删除团购商品sku集合
        activityTeamProductSkuInfoService.removeBatchByIds(activityTeamInfoUpdModel.getTeamProductSkuIds());
        //批量添加团购商品sku集合
        activityTeamProductSkuInfoService.saveBatch(
                //将activityTeamInfoUpdModel类型转成ActivityTeamProductSkuInfo类型集合
                activityTeamInfoUpdModel.getTeamProjectSkuInfoAddList().stream()
                        .map(ActivityTeamProductSkuInfo::buildAddSku)
                        .collect(Collectors.toList())
        );
        //根据activityTeamProductSkuList集合，批量修改团购商品sku集合
        List<ActivityTeamProductSkuUpdReq> activityTeamProductSkuList = activityTeamInfoUpdModel.getActivityTeamProductSkuList();
        activityTeamProductSkuInfoService.batchUpdate(
                activityTeamProductSkuList.stream()
                        .map(ActivityTeamProductSkuUpdModel::skuUpdBuild)
                        .collect(Collectors.toList())
        );

    }
}
