package com.muyu.product.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.base.CategoryBase;
import com.muyu.product.domain.model.CategoryInfoSaveModel;
import com.muyu.product.domain.model.TemplateAttributeGroupModel;
import com.muyu.product.domain.model.TemplateAttributeModel;
import com.muyu.product.domain.req.CategoryInfoUpdReq;
import com.muyu.product.domain.resp.CategoryCommonElementResp;
import com.muyu.product.domain.resp.CategoryInfoUpdResp;
import com.muyu.product.domain.resp.CategoryParentCommonElementResp;
import com.muyu.product.mapper.CategoryInfoMapper;
import com.muyu.product.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Stream;

/**
 * 品类信息Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class CategoryInfoServiceImpl extends ServiceImpl<CategoryInfoMapper, CategoryInfo> implements CategoryInfoService {


    @Autowired
    private AsCategoryAttributeService asCategoryAttributeService;

    @Autowired
    private AsCategoryAttributeGroupService asCategoryAttributeGroupService;

    @Autowired
    private AsCategoryBrandService asCategoryBrandService;

    @Autowired
    private AsAttributeGroupService asAttributeGroupService;

    @Autowired
    private AttributeInfoService attributeInfoService;

    @Autowired
    private BrandInfoService brandInfoService;

    @Autowired
    private AttributeGroupService attributeGroupService;
    @Resource
    private CategoryInfoMapper categoryInfoMapper;

    /**
     * 查询品类信息列表
     *
     * @param categoryInfo 品类信息
     *
     * @return 品类信息
     */
    @Override
    public List<CategoryInfo> list (CategoryInfo categoryInfo) {
        LambdaQueryWrapper<CategoryInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(categoryInfo.getName())) {
            queryWrapper.like(CategoryInfo::getName, categoryInfo.getName());
        }

        if (ObjUtils.notNull(categoryInfo.getImage())) {
            queryWrapper.eq(CategoryInfo::getImage, categoryInfo.getImage());
        }

        if (ObjUtils.notNull(categoryInfo.getParentId())) {
            queryWrapper.eq(CategoryInfo::getParentId, categoryInfo.getParentId());
        }

        if (ObjUtils.notNull(categoryInfo.getStart())) {
            queryWrapper.eq(CategoryInfo::getStart, categoryInfo.getStart());
        }

        if (ObjUtils.notNull(categoryInfo.getIntroduction())) {
            queryWrapper.eq(CategoryInfo::getIntroduction, categoryInfo.getIntroduction());
        }

        return list(queryWrapper);
    }

    /**
     * 保存商品品类
     * @param categoryInfoSaveModel 保存品类模型
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean save (CategoryInfoSaveModel categoryInfoSaveModel) {
        CategoryInfo categoryInfo = CategoryInfo.saveModelBuild(categoryInfoSaveModel);
        boolean save = this.save(categoryInfo);
        Long categoryInfoId = categoryInfo.getId();
        List<Long> attributeIdList = categoryInfoSaveModel.getAttributeIdList();
        if (attributeIdList != null && !attributeIdList.isEmpty()) {
            asCategoryAttributeService.saveBatch(
                    attributeIdList.stream()
                            .map(attributeId -> AsCategoryAttribute.categoryBuild(categoryInfoId, attributeId))
                            .toList()
            );
        }
        List<Long> attributeGroupIdList = categoryInfoSaveModel.getAttributeGroupIdList();
        if (attributeGroupIdList != null && !attributeGroupIdList.isEmpty()) {
            asCategoryAttributeGroupService.saveBatch(
                    attributeGroupIdList.stream()
                            .map(attributeGroupId -> AsCategoryAttributeGroup.categoryBuild(categoryInfoId, attributeGroupId))
                            .toList()
            );
        }
        List<Long> brandIdList = categoryInfoSaveModel.getBrandIdList();
        if (brandIdList != null && !brandIdList.isEmpty()) {
            asCategoryBrandService.saveBatch(
                    brandIdList.stream()
                            .map(brandId -> AsCategoryBrand.categoryBuild(categoryInfoId, brandId))
                            .toList()
            );
        }
        return save;
    }

    /**
     * 通过品类ID获取父级以上的属性组集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的属性组集合
     */
    @Override
    public List<AttributeGroup> getAttributeGroup (Long categoryId) {
        List<AttributeGroup> attributeGroupList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryAttributeGroup> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryAttributeGroup::getCategoryId, categoryId);
        List<AsCategoryAttributeGroup> asCategoryAttributeGroupList = asCategoryAttributeGroupService.list(queryWrapper);
        if (asCategoryAttributeGroupList != null && !asCategoryAttributeGroupList.isEmpty()) {
            List<Long> attributeGroupIdList = asCategoryAttributeGroupList.stream()
                    .map(AsCategoryAttributeGroup::getAttributeGroupId)
                    .toList();
            attributeGroupList.addAll(attributeGroupService.listByIds(attributeGroupIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (attributeGroupList.isEmpty()) {
                attributeGroupList.addAll(getAttributeGroup(categoryInfo.getParentId()));
            } else {
                List<AttributeGroup> attributeGroups = getAttributeGroup(categoryInfo.getParentId());
                attributeGroups.forEach(attributeGroup -> {
                    if (!attributeGroupList.contains(attributeGroup)) {
                        attributeGroupList.add(attributeGroup);
                    }
                });
            }
        }
        return attributeGroupList;
    }

    /**
     * 通过品类ID获取父级以上的品牌集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的品牌集合
     */
    @Override
    public List<BrandInfo> getBrand (Long categoryId) {
        List<BrandInfo> brandInfoList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryBrand> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryBrand::getCategoryId, categoryId);
        List<AsCategoryBrand> asCategoryBrandList = asCategoryBrandService.list(queryWrapper);
        if (asCategoryBrandList != null && !asCategoryBrandList.isEmpty()) {
            List<Long> brandIdList = asCategoryBrandList.stream()
                    .map(AsCategoryBrand::getBrandId)
                    .toList();
            brandInfoList.addAll(brandInfoService.listByIds(brandIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (brandInfoList.isEmpty()) {
                brandInfoList.addAll(getBrand(categoryInfo.getParentId()));
            } else {
                List<BrandInfo> brandInfos = getBrand(categoryInfo.getParentId());
                brandInfos.forEach(brandInfo -> {
                    if (!brandInfoList.contains(brandInfo)) {
                        brandInfoList.add(brandInfo);
                    }
                });
            }

        }
        return brandInfoList;
    }

    /**
     * 通过品类ID获取父级以上的属性集合
     * @param categoryId 品类ID
     * @return 父级以上的属性集合
     */
    @Override
    public List<AttributeInfo> getAttribute (Long categoryId) {
        List<AttributeInfo> attributeInfoList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryAttribute> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryAttribute::getCategoryId, categoryId);
        List<AsCategoryAttribute> asCategoryAttributeList = asCategoryAttributeService.list(queryWrapper);
        if (asCategoryAttributeList != null && !asCategoryAttributeList.isEmpty()) {
            List<Long> attributeIdList = asCategoryAttributeList.stream()
                    .map(AsCategoryAttribute::getAttributeId)
                    .toList();
            attributeInfoList.addAll(attributeInfoService.listByIds(attributeIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (attributeInfoList.isEmpty()) {
                attributeInfoList.addAll(getAttribute(categoryInfo.getParentId()));
            } else {
                List<AttributeInfo> attributeInfos = getAttribute(categoryInfo.getParentId());
                attributeInfos.forEach(attributeInfoQuery -> {
                    if (!attributeInfoList.contains(attributeInfoQuery)) {
                        attributeInfoList.add(attributeInfoQuery);
                    }
                });
            }
        }
        return attributeInfoList;
    }

    @Override
    public <T, AS> List<T> getCommon (Long categoryId, IService<AS> iService, IService<T> bsiService) {
        List<T> list = new ArrayList();
        QueryWrapper<AS> asQueryWrapper = new QueryWrapper<>();
        asQueryWrapper.eq("category_id", categoryId);
        List<AS> asList = iService.list(asQueryWrapper);
        if (asList != null && !asList.isEmpty()) {
            List<Long> baseIdList = asList.stream()
                    .map(as -> {
                        if (as instanceof CategoryBase categoryBase) {
                            return categoryBase.getBaseId();
                        }
                        return null;
                    })
                    .toList();
            list.addAll(bsiService.listByIds(baseIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (list.isEmpty()) {
                list.addAll(getCommon(categoryInfo.getParentId(), iService, bsiService));
            } else {
                List<T> commonList = getCommon(categoryInfo.getParentId(), iService, bsiService);
                commonList.forEach(common -> {
                    if (!list.contains(common)) {
                        list.add(common);
                    }
                });
            }
        }
        return list;
    }

    /**
     * 修改品类信息
     * @param id
     * @param categoryInfoUpdReq
     */
    @Override
    @Transactional  /**解决事务问题**/
    public Boolean updateById(Long id, CategoryInfoUpdReq categoryInfoUpdReq) {
        //修改品类信息
       Boolean b= categoryInfoMapper.updateById(id,categoryInfoUpdReq);
        //删除品类与属性，属性组，品牌关联表
        asCategoryAttributeService.remove(new LambdaQueryWrapper<AsCategoryAttribute>().eq(AsCategoryAttribute::getCategoryId,id));
        asCategoryAttributeGroupService.remove(new LambdaQueryWrapper<AsCategoryAttributeGroup>().eq(AsCategoryAttributeGroup::getCategoryId,id));
        asCategoryBrandService.remove(new LambdaQueryWrapper<AsCategoryBrand>().eq(AsCategoryBrand::getCategoryId,id));
        //添加新的关联数据(属性表)
        if(null!=categoryInfoUpdReq.getAttributeIdList()){
            categoryInfoUpdReq.getAttributeIdList().stream().forEach(attributeId -> {
                asCategoryAttributeService.addByGroup(attributeId,id);
            });
        }
        //添加新的关联数据(属性组表)
        if(null!=categoryInfoUpdReq.getAttributeGroupIdList()){
            categoryInfoUpdReq.getAttributeGroupIdList().stream().forEach(attributeGroupId -> {
               asCategoryAttributeGroupService.addByGroup(attributeGroupId,id);
            });
        }
        //添加新的关联数据(品类组表)
        if(null!=categoryInfoUpdReq.getBrandGroupIdList()){
            categoryInfoUpdReq.getBrandGroupIdList().stream().forEach(brandGroupId -> {
               asCategoryBrandService.addByGroup(brandGroupId,id);
            });
        }
        return b;
    }

    @Override
    public Result removeBatchById(Long ids) {
        //查询是否有子级
        //qw是plus的条件构造器，lqw是lambda表达式
        QueryWrapper<CategoryInfo> queryWrapper = new QueryWrapper<>();
        //根据父级id查询
        queryWrapper.eq("parent_id",ids);
        List<CategoryInfo> listall = categoryInfoMapper.selectList(queryWrapper);
        if(0==listall.size()){
            //是否有商品关联关系
           List<ProjectInfo> list1= categoryInfoMapper.selectProjectInfoById(ids);
           if(0==list1.size()){
               //删除
               categoryInfoMapper.delete(new LambdaQueryWrapper<CategoryInfo>().eq(CategoryInfo::getId,ids));
               return Result.success("删除成功！！！");
           }
           return Result.error("有相关商品关联关系，不能删除！！！");
        }
        return Result.error("有相关子集关系，不能删除！！！");
    }

    /**
     * 通过品类ID获取父级以上的属性、属性组、品牌集合
     * @param categoryId 品类ID
     * @return 父级以上的属性、属性组、品牌集合
     */
    @Override
    public CategoryParentCommonElementResp parentCommonElement (Long categoryId) {
        return CategoryParentCommonElementResp.builder()
                .attributeInfoList(getCommon(categoryId, asCategoryAttributeService, attributeInfoService))
                .attributeGroupList(getCommon(categoryId, asCategoryAttributeGroupService, attributeGroupService))
                .brandInfoList(getCommon(categoryId, asCategoryBrandService, brandInfoService))
                .build();
    }

    private void getParentIdListByCateGoryId(List<Long> parentIdList, Long cateGoryId){
        if (cateGoryId.equals(0L)){
            return;
        }
        CategoryInfo categoryInfo = this.getById(cateGoryId);
        parentIdList.add(categoryInfo.getId());
        getParentIdListByCateGoryId(parentIdList, categoryInfo.getParentId());
    }

    /**
     * 通过品类ID获取品类信息
     */
    @Override
    public CategoryInfoUpdResp getUpdById(Long id) {
        CategoryInfo categoryInfo = this.getById(id);
        //属性中间表构造器
        LambdaQueryWrapper<AsCategoryAttribute> AsAttributeQueryWrapper = new LambdaQueryWrapper<AsCategoryAttribute>().eq(AsCategoryAttribute::getCategoryId, id);
        List<AsCategoryAttribute> asCategoryAttributeList = asCategoryAttributeService.list(AsAttributeQueryWrapper);
        List<Long> longList = asCategoryAttributeList.stream().map(AsCategoryAttribute::getAttributeId).toList();
        //获取属性集合
        List<AttributeInfo> attributeInfoList = attributeInfoService.listByIds(longList);


        //属性组中间表构造器
        LambdaQueryWrapper<AsCategoryAttributeGroup> GroupQueryWrapper = new LambdaQueryWrapper<AsCategoryAttributeGroup>().eq(AsCategoryAttributeGroup::getCategoryId, id);
        List<AsCategoryAttributeGroup> asCategoryAttributeGroupList = asCategoryAttributeGroupService.list(GroupQueryWrapper);
        List<Long> longList1 = asCategoryAttributeGroupList.stream().map(AsCategoryAttributeGroup::getAttributeGroupId).toList();
        //获取属性组集合
        List<AttributeGroup> attributeGroups = attributeGroupService.listByIds(longList1);

        //品牌中间表构造器
        LambdaQueryWrapper<AsCategoryBrand> brandQueryWrapper = new LambdaQueryWrapper<AsCategoryBrand>().eq(AsCategoryBrand::getCategoryId, id);
        List<AsCategoryBrand> asCategoryBrandList = asCategoryBrandService.list(brandQueryWrapper);
        List<Long> longList2 = asCategoryBrandList.stream().map(AsCategoryBrand::getBrandId).toList();
        List<BrandInfo> brandInfoList = brandInfoService.listByIds(longList2);

        return CategoryInfoUpdResp.builder()
                .id(id)
                .name(categoryInfo.getName())
                .image(categoryInfo.getImage())
                .start(categoryInfo.getStart())
                .introduction(categoryInfo.getIntroduction())
                .attributeInfoList(attributeInfoList)
                .attributeGroupList(attributeGroups)
                .brandInfoList(brandInfoList)
                .build();
    }

    /**
     * 通过品类ID获取品类共有属性
     * @return 品类共有属性
     */
    @Override
    public CategoryCommonElementResp getTemplateAttributeByCateGoryId (Long cateGoryId) {
        List<Long> cateGoryIdList = new ArrayList<>();
        getParentIdListByCateGoryId(cateGoryIdList, cateGoryId);
        // 获取和品类相互关联的属性组关系  （中间表）
        LambdaQueryWrapper<AsCategoryAttributeGroup> asCategoryAttributeGroupLambdaQueryWrapper = new LambdaQueryWrapper<>(){{
            in(AsCategoryAttributeGroup::getCategoryId, cateGoryIdList);
        }};
        List<AsCategoryAttributeGroup> categoryAttributeGroupList = asCategoryAttributeGroupService.list(asCategoryAttributeGroupLambdaQueryWrapper);
        List<TemplateAttributeGroupModel>  attributeGroupModelList = categoryAttributeGroupList.stream()
                .map(AsCategoryAttributeGroup::getAttributeGroupId)
                .distinct()
                .map(attributeGroupId -> TemplateAttributeGroupModel.attributeGroupBuild(
                        attributeGroupService.getById(attributeGroupId),
                        applyAttributeGroupId -> {
                            LambdaQueryWrapper<AsAttributeGroup> asAttributeGroupQueryWrapper = new LambdaQueryWrapper<>() {{
                                eq(AsAttributeGroup::getGroupId, applyAttributeGroupId);
                            }};
                            List<Long> attributeIdList = asAttributeGroupService.list(asAttributeGroupQueryWrapper).stream()
                                    .map(AsAttributeGroup::getAttributeId)
                                    .toList();
                            if (attributeIdList.isEmpty()) {
                                return new ArrayList<>();
                            }
                            return attributeInfoService.listByIds(attributeIdList).stream()
                                    .map(AttributeInfo::buildTemplateModel)
                                    .toList();
                        }
                ))
                .filter(TemplateAttributeGroupModel::isEffective)
                .toList();
        // 创建集合
        Set<Long> attributeIdSet = new HashSet<>();
//        attributeGroupModelList.forEach(attributeGroupModel -> {
//            attributeGroupModel.getAttributeList().forEach(attributeModel -> {
//                attributeIdSet.add(attributeModel.getId());
//            });
//        });
        // 获取组内所有属性Id，判空
        if (!attributeGroupModelList.isEmpty()){
            attributeIdSet.addAll(
                    attributeGroupModelList.stream()
                            .flatMap((Function<TemplateAttributeGroupModel, Stream<TemplateAttributeModel>>) templateAttributeGroupModel -> templateAttributeGroupModel.getAttributeList().stream())
                            .map(TemplateAttributeModel::getId)
                            .toList()
        );
    }

        List<TemplateAttributeModel> templateAttributeModelList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryAttribute> categoryAttributeQueryWrapper = new LambdaQueryWrapper<>();
        categoryAttributeQueryWrapper.in(AsCategoryAttribute::getCategoryId, cateGoryIdList);
        List<AsCategoryAttribute> asCategoryAttributeList = asCategoryAttributeService.list(categoryAttributeQueryWrapper);
        if (asCategoryAttributeList != null && !asCategoryAttributeList.isEmpty()){
            List<Long> templateAttributeIdList = asCategoryAttributeList.stream()
                    .map(AsCategoryAttribute::getAttributeId)
                    .filter(templateAttributeId -> !attributeIdSet.contains(templateAttributeId))
                    .toList();
            if (!templateAttributeIdList.isEmpty()){
                templateAttributeModelList = attributeInfoService.listByIds(templateAttributeIdList).stream()
                        .map(AttributeInfo::buildTemplateModel)
                        .toList();
            }

        }

        List<TemplateAttributeModel> attributeModelList = new ArrayList<>();
        //判断是否为空，不为空继续执行
        if (!templateAttributeModelList.isEmpty()){
            attributeIdSet.addAll(
                    //利用Steam 流遍历获取id集合
                    templateAttributeModelList.stream().map(TemplateAttributeModel::getId).toList()
            );
        }

        //商品属性构造
        LambdaQueryWrapper<AttributeInfo> attributeInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
        attributeInfoLambdaQueryWrapper.notIn(!attributeIdSet.isEmpty(), AttributeInfo::getId, attributeIdSet);
        attributeModelList = attributeInfoService.list(attributeInfoLambdaQueryWrapper).stream().map(AttributeInfo::buildTemplateModel).toList();

        //属性组列表回显（商品信息中）
        return CategoryCommonElementResp.builder()
                .templateAttributeGroupList(attributeGroupModelList)
                .templateAttributeList(templateAttributeModelList)
                .attributeList(attributeModelList)
                .build();
    }
}
